"""
The InvenTreeBarcodePlugin validates barcodes generated by InvenTree itself.
It can be used as a template for developing third-party barcode plugins.

The data format is very simple, and maps directly to database objects,
via the "id" parameter.

Parsing an InvenTree barcode simply involves validating that the
references model objects actually exist in the database.
"""

# -*- coding: utf-8 -*-

import json

from barcode.barcode import BarcodePlugin

from stock.models import StockItem, StockLocation
from part.models import Part

from rest_framework.exceptions import ValidationError


class InvenTreeBarcodePlugin(BarcodePlugin):

    PLUGIN_NAME = "InvenTreeBarcode"

    def validate(self):
        """
        An "InvenTree" barcode must be a jsonnable-dict with the following tags:

        {
            'tool': 'InvenTree',
            'version': <anything>
        }

        """

        # The data must either be dict or be able to dictified
        if type(self.data) is dict:
            pass
        elif type(self.data) is str:
            try:
                self.data = json.loads(self.data)
            except json.JSONDecodeError:
                return False
        else:
            return False

        for key in ['tool', 'version']:
            if key not in self.data.keys():
                return False

        if not self.data['tool'] == 'InvenTree':
            return False

        return True

    def getStockItem(self):

        for k in self.data.keys():
            if k.lower() == 'stockitem':
                try:
                    pk = self.data[k]['id']
                except (AttributeError, KeyError):
                    raise ValidationError({k: "id parameter not supplied"})

                try:
                    item = StockItem.objects.get(pk=pk)
                    return item
                except (ValueError, StockItem.DoesNotExist):
                    raise ValidationError({k, "Stock item does not exist"})

        return None

    def getStockLocation(self):

        for k in self.data.keys():
            if k.lower() == 'stocklocation':
                try:
                    pk = self.data[k]['id']
                except (AttributeError, KeyError):
                    raise ValidationError({k: "id parameter not supplied"})

                try:
                    loc = StockLocation.objects.get(pk=pk)
                    return loc
                except (ValueError, StockLocation.DoesNotExist):
                    raise ValidationError({k, "Stock location does not exist"})

        return None

    def getPart(self):

        for k in self.data.keys():
            if k.lower() == 'part':
                try:
                    pk = self.data[k]['id']
                except (AttributeError, KeyError):
                    raise ValidationError({k, 'id parameter not supplied'})

                try:
                    part = Part.objects.get(pk=pk)
                    return part
                except (ValueError, Part.DoesNotExist):
                    raise ValidationError({k, 'Part does not exist'})

        return None
